//*******************************************************
//
//	Color LCD Clock
//
//	液晶表示関係()
//
//	ATmega1284p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	JTAG禁止, EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.4
//				AVR-LIBC	1.6.7
//				binutils	2.20
//
//	2011.5.22	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"
#include	"lcd.h"
#include	"LCDfont.h"

#include	<util/delay.h>

uchar	CurX, CurY;
uint	Color, BGcolor;

// LCD初期化データ
PROGMEM unsigned char LCD_init_data[] = {

	0x01, 0x02,		// 128x160 16/18 bit color + partial
	0x02, 0x12,		// V, H normal scan
	0x03, 0x20,		// 8bit mode, Transfer 16bit data(R:G:B = 5:6:5)
	0x05, 0x08,		// VCO_Mode
	0x07, 0x7f,		// VCOMH_CTRL
	0x08, 0x17,		// VCOML_CTRL
	0x17, 0x00,		// SRAM_Control
	0x09, 0x00,		// Write SRAM window start X point
	0x10, 0x00,		// Write SRAM window start Y point
	0x11, 0x7F,		// Write SRAM window end X point
	0x12, 0x9f,		// Write SRAM window end Y point
	0x14, 0x00,		// Pertial Display start Y point
	0x16, 0x9f,		// Pertial Display end Y point
	0x18, 0x00,		// SRAM X position
	0x19, 0x00,		// SRAM Y position
	0x21, 0x88,		// Gamma adjust ~0x29
	0x22, 0x33,
	0x23, 0xcc,
	0x24, 0xcc,
	0x25, 0xbb,
	0x26, 0xbb,
	0x27, 0x77,
	0x28, 0x77,
	0x29, 0x77,
	0x06, 0xc7,		// DAC_OP_CTRL2

	0x00
};


//*******************************************************
//	矩形塗りつぶし
//*******************************************************
void fill_rect(uchar xs, uchar ys, uchar xe, uchar ye, uint color){
	int	x, y;

	set_window(xs, ys, xe, ye);

	PORTA &= ~(1<<LCS);
	for (y = ys; y <= ye; y++){
		for (x = xs; x <= xe; x++){
			PORTA &= ~(1<<LWR);
			PORTC = color >> 8;
			PORTA |= (1<<LWR);

			PORTA &= ~(1<<LWR);
			PORTC = color;
			PORTA |= (1<<LWR);
		}
	}
	PORTA |= (1<<LCS);
}


//*******************************************************
//	矩形枠表示
//*******************************************************
void rectangle(uchar xs, uchar ys, uchar xe, uchar ye, uint color){

	fill_rect(xs, ys, xe, ys, color);
	fill_rect(xs, ye, xe, ye, color);
	fill_rect(xs, ys+1, xs, ye-1, color);
	fill_rect(xe, ys+1, xe, ye-1, color);
}


//*******************************************************
//	文字列表示
//*******************************************************
void print(char *str){
	char	c;

	while((c = *str++)){
		putch(c);
	}
}


//*******************************************************
//	固定文字列表示
//*******************************************************
void print_P(PGM_VOID_P str){
	char	c;

	while((c = pgm_read_byte(str++))){
		putch(c);
	}
}


//*******************************************************
//	カーソル位置に指定した色の半角文字描画
//*******************************************************
void putch(char ch){

	if (ch == CR){
		CurX = 0;
	} else if (ch == LF){
		CurX = 0;
		if (CurY < LCD_HEIGHT)	CurY += 8;
	} else {
		if ((ch < ' ')||(ch > 0x7F)){
			ch = '.';
		}
		if (CurX > (LCD_WIDTH - 6)){
			CurX = 0;
			if (CurY < LCD_HEIGHT)	CurY += 8;
		}
		if (CurY < (LCD_HEIGHT - 7)){
			PGM_VOID_P	ps;
			PGM_VOID_P	p;
			uchar	i, mask;

			set_window(CurX, CurY, CurX + 5, CurY + 7);

			ps = (PGM_VOID_P)&AsciiFont[(ch - 0x20) * 5];

			PORTA &= ~(1<<LCS);
			for (mask = 1; mask; mask <<= 1){
				p = ps;
				for (i = 0; i < 5; i++){
					if (pgm_read_byte(p++) & mask){
						PORTA &= ~(1<<LWR);
						PORTC = Color >> 8;
						PORTA |= (1<<LWR);
						PORTA &= ~(1<<LWR);
						PORTC = Color;
						PORTA |= (1<<LWR);
					} else {
						PORTA &= ~(1<<LWR);
						PORTC = BGcolor >> 8;
						PORTA |= (1<<LWR);
						PORTA &= ~(1<<LWR);
						PORTC = BGcolor;
						PORTA |= (1<<LWR);
					}
				}
				PORTA &= ~(1<<LWR);
				PORTC = BGcolor >> 8;
				PORTA |= (1<<LWR);
				PORTA &= ~(1<<LWR);
				PORTC = BGcolor;
				PORTA |= (1<<LWR);
			}
			PORTA |= (1<<LCS);
			CurX += 6;
		}
	}
}


//*******************************************************
//	カーソル位置設定
//*******************************************************
void locate(int x, int y){

	CurX = x;
	CurY = y;
}


//*******************************************************
//	文字色設定
//*******************************************************
void color(uint color){

	Color = color;
}


//*******************************************************
//	文字の背景色設定
//*******************************************************
void back_color(uint color){

	BGcolor = color;
}


//*******************************************************
//	LCDクリア
//*******************************************************
void clear_LCD(void){

	CurX = CurY = 0;
	fill_rect(0, 0, LCD_WIDTH-1, LCD_HEIGHT-1, BGcolor);
}


//*******************************************************
//	LCD Window set
//*******************************************************
void set_window(uchar xs, uchar ys, uchar xe, uchar ye){

	LCD_com_data(0x09, xs);
	LCD_com_data(0x10, ys);
	LCD_com_data(0x11, xe);
	LCD_com_data(0x12, ye);

	LCD_com_data(0x18, xs);
	LCD_com_data(0x19, ys);
}



//*******************************************************
//	LCD コマンドデータ
//*******************************************************
void LCD_com_data(uchar com, uchar data){

	DDRC = 0xFF;
	PORTA &= ~(1<<LRS);
	PORTA &= ~(1<<LCS);
	PORTA &= ~(1<<LWR);
	PORTC = com;
	PORTA |= (1<<LWR);
	PORTA |= (1<<LRS);
	PORTA &= ~(1<<LWR);
	PORTC = data;
	PORTA |= (1<<LWR);
	PORTA |= (1<<LCS);
}


//*******************************************************
//	LCD初期化
//*******************************************************
void LCD_init(void){
	PGM_VOID_P	p;
	uchar	adrs, data;

	PORTC = 0;
	DDRC = 0xFF;

	_delay_ms(5);
	PORTA &= ~(1<<LRES);
	_delay_ms(10);
	PORTA |= (1<<LRES);
	_delay_ms(20);

	p = (PGM_VOID_P)LCD_init_data;
	while((adrs = pgm_read_byte(p++)) != 0){
		data = pgm_read_byte(p++);
		LCD_com_data(adrs, data);
	}

	CurX = CurY = 0;
	Color = COLOR_WHITE;
	BGcolor = COLOR_BLACK;

	clear_LCD();
}

