#ifndef	__MAIN_H__
#define	__MAIN_H__
//*******************************************************
//
//	Color LCD Clock
//
//	ATmega1284p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	JTAG禁止, EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.4
//				AVR-LIBC	1.6.7
//				binutils	2.20
//
//	2011.5.24	KAWAKAMI Yukio
//*******************************************************

#include	<avr/pgmspace.h>
#include	"pff.h"

#define		VERSION		"1.0.7"

#define		F_CPU	7987200UL	// 38400 x 16 x 13

#define		BACKLIGHT_TIME	30	// バックライト点灯時間

#define		ADRS_DATA	0x08000L		// Flashデータ格納エリア
#define		ADRS_LIMIT	0x1DC00L		// データ格納エリア上限＋１
#define		ADRS_BOOT	0x1E000L		// ブートローダスタートアドレス
#define		ADRS_WRITE_FLASH	0x1FF40	// Flash書き込み関数アドレス

#define		CR	13
#define		LF	10
#define		TAB	9
#define		ON	1
#define		OFF	0
#define		TRUE	1
#define		FALSE	0

#define		NOP		asm volatile("nop\n"::)
#define		WDR		asm volatile("wdr\n"::)
#define		SLEEP	asm volatile("sleep\n"::)

typedef	unsigned char	uchar;
typedef	unsigned int	uint;
typedef	unsigned long	ulong;
typedef unsigned short	ushort;


#define		LED2	7	// PORTA
#define		LED1	6
#define		RSTB	5
#define		LWR		4
#define		LRD		3
#define		LRES	2
#define		LRS		1
#define		LCS		0

#define		SCK		7	// PORTB
#define		MISO	6
#define		MOSI	5
#define		SDCS	4
#define		RDATA	3
#define		RCLK	2
#define		CCE		1
#define		MCS		0

#define		LCD_DB_OUT	PORTC
#define		LCD_DB_IN	PINC

#define		BCE		6	// PORTD
#define		BR2		5
#define		BR1		4
#define		C1HZ	3
#define		RWR		2
#define		TXD		1
#define		RXD		0


// RTC-4543 レジスタ 各々4bit
#define	SEC_LOW		0
#define	SEC_HIGH	1
#define	MIN_LOW		2
#define	MIN_HIGH	3
#define	HOUR_LOW	4
#define	HOUR_HIGH	5
#define	WEEK		6
#define	DAY_LOW		7
#define	DAY_HIGH	8
#define	MONTH_LOW	9
#define	MONTH_HIGH	10
#define	YEAR_LOW	11
#define	YEAR_HIGH	12


typedef struct tagGRPHEADER {
	unsigned short	mark;
	short	width;
	short	height;
	short	type;
	short	comp1;
	short	comp2;
	unsigned char yobi[20];
} GRPHEADER;

struct RTC_TIME {
	uchar	sec;
	uchar	min;
	uchar	hour;
	uchar	week;
	uchar	day;
	uchar	month;
	uchar	year;
};

union DATETIME_PACK {
	struct unpack_s {
		uint32_t	year:7;
		uint32_t	month:4;
		uint32_t	day:5;
		uint32_t	hour:5;
		uint32_t	min:6;
		uint32_t	sec:5;
	} unpack;
	ulong	pack;
};


#define	EEPROM_SELF_PROGRAM	(uint8_t *)0
#define	EEPROM_FIRM_FTIME	(uint16_t *)1	// 1,2
#define	EEPROM_FIRM_FDATE	(uint16_t *)3	// 3,4

void job_serial(void);

char get_fdatetime(char *fname, ushort *fdate, ushort *ftime);

char load_grp(char *fname, uchar x, uchar y);

void load_back(void);
void put_grp(ulong adrs, uchar x, uchar y);
void pgm_read_block(ulong adrs, uchar *buff, uint size);

void set_time(void);
uchar set_time_sub(uchar x, uchar y, uchar *n, uchar min, uchar max);

void write_flash(ulong adrs, uchar data);

uchar check_sw(void);

uchar SPI_in(void);
void SPI_out(uchar data);
uchar read_ram(uint adrs);
void read_ram_block(uint adrs, uchar *buff, int size);
void write_ram(uint adrs, uchar data);
void write_ram_block(uint adrs, uchar *buff, int size);

void set_RTC(struct RTC_TIME *rtc);
void read_RTC(struct RTC_TIME *rtc);
void set_week(struct RTC_TIME *rtc);
void add_sec(struct RTC_TIME *rtc, uchar sec);

void wait_timer(uchar tm);

#endif	// __MAIN_H__
